# -*- coding: utf-8 -*-

"""
The module contains some event filters that can be useful

"""

import weakref

from ._base import MplEventDispatcher, MplEvent
from ._types import MplEvent_Type
from ._logging import logger


class LoggingEventFilter:
    """Logging events

    The filter logs all events that will be handled.

    .. note::

        This event filter should be append to end of of event filters list.

    """

    def __call__(self, obj: MplEventDispatcher, event: MplEvent_Type):
        logger.info('%s (%s) %s', obj, MplEvent(event.name), event)


class MouseDoubleClickReleaseEventFilter:
    """Sets correct value of dblclick flag to MOUSE_BUTTON_RELEASE event

    The filter adds  true ``dblclick`` flag to ``MOUSE_BUTTON_RELEASE`` event
    if ``dblclick`` is True in previous ``MOUSE_BUTTON_PRESS`` event.
    """

    def __init__(self):
        self._double_clicks = weakref.WeakKeyDictionary()

    def __call__(self, obj: MplEventDispatcher, event: MplEvent_Type):
        event_type = MplEvent(event.name)

        if event_type not in {MplEvent.MOUSE_BUTTON_PRESS,
                              MplEvent.MOUSE_BUTTON_RELEASE}:
            return

        if event_type == MplEvent.MOUSE_BUTTON_PRESS:
            self._double_clicks[obj] = event.dblclick

        if event_type == MplEvent.MOUSE_BUTTON_RELEASE:
            if obj in self._double_clicks and self._double_clicks[obj]:
                self._double_clicks[obj] = False
                event.dblclick = True



# -*- coding: utf-8 -*-

import matplotlib.axes as _mpl_ax
import matplotlib.figure as _mpl_fig
import matplotlib.backend_bases as _mpl_bb

FigureCanvas = _mpl_bb.FigureCanvasBase
Figure = _mpl_fig.Figure
Axes = _mpl_ax.Axes

Event = _mpl_bb.Event
KeyEvent = _mpl_bb.KeyEvent
MouseEvent = _mpl_bb.MouseEvent
PickEvent = _mpl_bb.PickEvent
LocationEvent = _mpl_bb.LocationEvent
ResizeEvent = _mpl_bb.ResizeEvent
CloseEvent = _mpl_bb.CloseEvent
DrawEvent = _mpl_bb.DrawEvent



# -*- coding: utf-8 -*-

import typing as t

from .mpl import (
    FigureCanvas,
    Figure,
    Axes,

    KeyEvent,
    MouseEvent,
    PickEvent,
    LocationEvent,
    ResizeEvent,
    CloseEvent,
    DrawEvent,
)

MplEvent_Type = t.Union[KeyEvent, MouseEvent, PickEvent, LocationEvent, ResizeEvent, CloseEvent, DrawEvent]
MplObject_Type = t.Union[Axes, Figure, FigureCanvas]
EventHandler_Type = t.Callable[[MplEvent_Type], None]
EventFilter_Type = t.Callable[['MplEventDispatcher', MplEvent_Type], t.Optional[bool]]
WeakRefFigure_Type = t.Optional[Figure]
